@@ -1,4 +1,3 @@
-
 #include <filezilla.h>
 #include "Mainfrm.h"
 
@@ -53,8 +52,6 @@
 #include "welcome_dialog.h"
 #include "window_state_manager.h"
 
-#include <wx/tokenzr.h>
-
 #ifdef __WXMSW__
 #include <wx/module.h>
 #endif
@@ -65,6 +62,7 @@
 #endif
 
 #include <functional>
+#include <limits>
 #include <map>
 
 #ifdef __WXGTK__
@@ -199,11 +197,11 @@ public:
 	}
 
 protected:
-	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, const wxString&, const void*)
+	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, std::wstring const&, const void*) override
 	{
 		if (notification == STATECHANGE_CHANGEDCONTEXT) {
 			// Update window title
-			if (!pState || !pState->GetServer()) {
+			if (!pState || !pState->GetSite()) {
 				m_pMainFrame->SetTitle(_T("FileZilla"));
 			}
 			else {
@@ -235,10 +233,9 @@ protected:
 		}
 
 		if (notification == STATECHANGE_SERVER) {
-			ServerWithCredentials const& server = pState->GetServer();
-
 			if (pState == CContextManager::Get()->GetCurrentContext()) {
-				if (!server) {
+				Site const& site = pState->GetSite();
+				if (!site) {
 					m_pMainFrame->SetTitle(_T("FileZilla"));
 				}
 				else {
@@ -460,6 +457,7 @@ CMainFrame::CMainFrame()
 		m_pBottomSplitter->Initialize(m_pContextControl);
 	}
 
+	wxGetApp().AddStartupProfileRecord("CMainFrame::CMainFrame pre layout");
 	m_pWindowStateManager = new CWindowStateManager(this);
 	m_pWindowStateManager->Restore(OPTION_MAINWINDOW_POSITION);
 
@@ -607,16 +605,16 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		OpenSiteManager();
 	}
 	else if (event.GetId() == XRCID("ID_MENU_FILE_COPYSITEMANAGER")) {
-		ServerWithCredentials server;
+		Site site;
 		CState* pState = CContextManager::Get()->GetCurrentContext();
 		if (pState) {
-			server = pState->GetServer();
+			site = pState->GetSite();
 		}
-		if (!server) {
+		if (!site) {
 			wxMessageBoxEx(_("Not connected to any server."), _("Cannot add server to Site Manager"), wxICON_EXCLAMATION);
 			return;
 		}
-		OpenSiteManager(&server);
+		OpenSiteManager(&site);
 	}
 	else if (event.GetId() == XRCID("ID_MENU_SERVER_CMD")) {
 		CState* pState = CContextManager::Get()->GetCurrentContext();
@@ -850,7 +848,7 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		}
 
 		Site const old_site = pState->GetSite();
-		std::wstring sitePath = old_site.m_path;
+		std::wstring sitePath = old_site.SitePath();
 
 		CContextControl::_context_controls* controls = m_pContextControl->GetCurrentControls();
 		if (!controls) {
@@ -860,11 +858,11 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		// controls->last_bookmark_path can get modified if it's empty now
 		int res;
 		if (event.GetId() == XRCID("ID_BOOKMARK_ADD")) {
-			CNewBookmarkDialog dlg(this, sitePath, old_site.server_ ? &old_site.server_ : 0);
+			CNewBookmarkDialog dlg(this, sitePath, old_site ? &old_site : 0);
 			res = dlg.Run(pState->GetLocalDir().GetPath(), pState->GetRemotePath());
 		}
 		else {
-			CBookmarksDialog dlg(this, sitePath, old_site.server_ ? &old_site.server_ : 0);
+			CBookmarksDialog dlg(this, sitePath, old_site ? &old_site : 0);
 			res = dlg.Run();
 		}
 		if (res == wxID_OK) {
@@ -874,7 +872,7 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 					for (int i = 0; i < m_pContextControl->GetTabCount(); ++i) {
 						CContextControl::_context_controls *tab_controls = m_pContextControl->GetControlsFromTabIndex(i);
 						if (tab_controls) {
-							tab_controls->pState->UpdateSite(old_site.m_path, *site);
+							tab_controls->pState->UpdateSite(old_site.SitePath(), *site);
 						}
 					}
 				}
@@ -1191,7 +1189,7 @@ bool CMainFrame::CloseDialogsAndQuit(wxCloseEvent &event)
 			}
 
 			wxDialog* pDialog = dynamic_cast<wxDialog*>(pTop);
-			if (pDialog) {
+			if (pDialog && pDialog->IsModal()) {
 				pDialog->EndModal(wxID_CANCEL);
 			}
 			else {
@@ -1363,8 +1361,12 @@ void CMainFrame::OnClose(wxCloseEvent &event)
 	if (m_pContextControl) {
 		CContextControl::_context_controls* controls = m_pContextControl->GetCurrentControls();
 		if (controls) {
-			controls->pLocalListView->SaveColumnSettings(OPTION_LOCALFILELIST_COLUMN_WIDTHS, OPTION_LOCALFILELIST_COLUMN_SHOWN, OPTION_LOCALFILELIST_COLUMN_ORDER);
-			controls->pRemoteListView->SaveColumnSettings(OPTION_REMOTEFILELIST_COLUMN_WIDTHS, OPTION_REMOTEFILELIST_COLUMN_SHOWN, OPTION_REMOTEFILELIST_COLUMN_ORDER);
+			if (controls->pLocalListView) {
+				controls->pLocalListView->SaveColumnSettings(OPTION_LOCALFILELIST_COLUMN_WIDTHS, OPTION_LOCALFILELIST_COLUMN_SHOWN, OPTION_LOCALFILELIST_COLUMN_ORDER);
+			}
+			if (controls->pRemoteListView) {
+				controls->pRemoteListView->SaveColumnSettings(OPTION_REMOTEFILELIST_COLUMN_WIDTHS, OPTION_REMOTEFILELIST_COLUMN_SHOWN, OPTION_REMOTEFILELIST_COLUMN_ORDER);
+			}
 		}
 
 		m_pContextControl->SaveTabs();
@@ -1436,7 +1438,7 @@ void CMainFrame::OnTimer(wxTimerEvent& event)
 #endif
 }
 
-void CMainFrame::OpenSiteManager(ServerWithCredentials const* pServer)
+void CMainFrame::OpenSiteManager(Site const* site)
 {
 	CState* pState = CContextManager::Get()->GetCurrentContext();
 	if (!pState) {
@@ -1445,55 +1447,55 @@ void CMainFrame::OpenSiteManager(ServerWithCredentials const* pServer)
 
 	CSiteManagerDialog dlg;
 
-	std::set<std::wstring> handled_paths;
 	std::vector<CSiteManagerDialog::_connected_site> connected_sites;
 
-	if (pServer) {
+	if (site) {
 		CSiteManagerDialog::_connected_site connected_site;
-		connected_site.server = *pServer;
+		connected_site.site = *site;
 		connected_sites.push_back(connected_site);
 	}
 
 	for (int i = 0; i < m_pContextControl->GetTabCount(); ++i) {
 		CContextControl::_context_controls *controls =  m_pContextControl->GetControlsFromTabIndex(i);
-		if (!controls) {
+		if (!controls || !controls->pState) {
 			continue;
 		}
 
-		Site const& site = controls->pState->GetSite();
-		std::wstring const& path = site.m_path;
+		Site site = controls->pState->GetSite();
+		if (!site) {
+			site = controls->pState->GetLastSite();
+		}
+
+		std::wstring const& path = site.SitePath();
 		if (path.empty()) {
 			continue;
 		}
-		if (handled_paths.find(path) != handled_paths.end()) {
+
+		auto it = std::find_if(connected_sites.cbegin(), connected_sites.cend(), [&](auto const& v) { return v.old_path == path; });
+		if (it != connected_sites.cend()) {
 			continue;
 		}
 
 		CSiteManagerDialog::_connected_site connected_site;
 		connected_site.old_path = path;
-		connected_site.server = site.server_;
+		connected_site.site = site;
 		connected_sites.push_back(connected_site);
-		handled_paths.insert(path);
 	}
 
-	if (!dlg.Create(this, &connected_sites, pServer)) {
+	if (!dlg.Create(this, &connected_sites, site)) {
 		return;
 	}
 
 	int res = dlg.ShowModal();
 	if (res == wxID_YES || res == wxID_OK) {
 		// Update bookmark paths
-		for (auto const& connected_site : connected_sites) {
-			std::unique_ptr<Site> site = CSiteManager::GetSiteByPath(connected_site.new_path, false).first;
-			if (site) {
-				for (int i = 0; i < m_pContextControl->GetTabCount(); ++i) {
-					CContextControl::_context_controls *controls = m_pContextControl->GetControlsFromTabIndex(i);
-					if (!controls) {
-						continue;
-					}
-					controls->pState->UpdateSite(connected_site.old_path, *site);
-				}
+		for (int i = 0; i < m_pContextControl->GetTabCount(); ++i) {
+			CContextControl::_context_controls *controls = m_pContextControl->GetControlsFromTabIndex(i);
+			if (!controls || !controls->pState) {
+				continue;
 			}
+
+			controls->pState->UpdateKnownSites(connected_sites);
 		}
 	}
 
@@ -1766,7 +1768,7 @@ void CMainFrame::UpdaterStateChanged(UpdaterState s, build const& v)
 		return;
 	}
 
-	if( s == UpdaterState::idle ) {
+	if (s == UpdaterState::idle) {
 		wxMenu* m = 0;
 		wxMenuItem* pItem = m_pMenuBar->FindItem(GetAvailableUpdateMenuId(), &m);
 		if (pItem && m) {
@@ -2025,7 +2027,7 @@ void CMainFrame::OnSitemanagerDropdown(wxCommandEvent& event)
 bool CMainFrame::ConnectToSite(Site & data, Bookmark const& bookmark, CState* pState)
 {
 	// First check if we need to ask user for a password
-	if (!CLoginManager::Get().GetPassword(data.server_, false, data.server_.server.GetName())) {
+	if (!CLoginManager::Get().GetPassword(data, false)) {
 		return false;
 	}
 
@@ -2070,7 +2072,9 @@ bool CMainFrame::ConnectToSite(Site & data, Bookmark const& bookmark, CState* pS
 		}
 
 		if (!(action & 1)) {
-			m_pContextControl->CreateTab();
+			if (!m_pContextControl->CreateTab()) {
+				return false;
+			}
 			pState = CContextManager::Get()->GetCurrentContext();
 		}
 	}
@@ -2211,29 +2215,29 @@ void CMainFrame::RememberSplitterPositions()
 		return;
 	}
 
-	wxString posString;
+	std::wstring const posString = fz::sprintf(
+		L"%d %d %d %d %d %d",
+		// top_pos
+		m_pTopSplitter->GetSashPosition(),
 
-	// top_pos
-	posString += wxString::Format(_T("%d "), m_pTopSplitter->GetSashPosition());
+		// bottom_height
+		m_pBottomSplitter->GetSashPosition(),
 
-	// bottom_height
-	posString += wxString::Format(_T("%d "), m_pBottomSplitter->GetSashPosition());
+		// view_pos
+		// Note that we cannot use %f, it is locale-dependent
+		static_cast<int>(controls->pViewSplitter->GetRelativeSashPosition() * 1000000000),
 
-	// view_pos
-	posString += wxString::Format(_T("%d "), (int)(controls->pViewSplitter->GetRelativeSashPosition() * 1000000000));
+		// local_pos
+		controls->pLocalSplitter->GetSashPosition(),
 
-	// local_pos
-	posString += wxString::Format(_T("%d "), controls->pLocalSplitter->GetSashPosition());
+		// remote_pos
+		controls->pRemoteSplitter->GetSashPosition(),
 
-	// remote_pos
-	posString += wxString::Format(_T("%d "), controls->pRemoteSplitter->GetSashPosition());
+		// queuelog splitter
+		static_cast<int>(m_pQueueLogSplitter->GetRelativeSashPosition() * 1000000000)
+	);
 
-	// queuelog splitter
-	// Note that we cannot use %f, it is locale-dependent
-	// m_lastQueueLogSplitterPos is a value between 0 and 1
-	posString += wxString::Format(_T("%d"), (int)(m_pQueueLogSplitter->GetRelativeSashPosition() * 1000000000));
-
-	COptions::Get()->SetOption(OPTION_MAINWINDOW_SPLITTER_POSITION, posString.ToStdWstring());
+	COptions::Get()->SetOption(OPTION_MAINWINDOW_SPLITTER_POSITION, posString);
 }
 
 bool CMainFrame::RestoreSplitterPositions()
@@ -2243,43 +2247,40 @@ bool CMainFrame::RestoreSplitterPositions()
 	}
 
 	// top_pos bottom_height view_pos view_height_width local_pos remote_pos
-	wxString posString = COptions::Get()->GetOption(OPTION_MAINWINDOW_SPLITTER_POSITION);
-	wxStringTokenizer tokens(posString, _T(" "));
-	int count = tokens.CountTokens();
-	if (count < 6)
+	auto tokens = fz::strtok(COptions::Get()->GetOption(OPTION_MAINWINDOW_SPLITTER_POSITION), L" ");
+	if (tokens.size() < 6) {
 		return false;
+	}
 
-	long * aPosValues = new long[count];
-	for (int i = 0; i < count; ++i) {
-		wxString token = tokens.GetNextToken();
-		if (!token.ToLong(aPosValues + i)) {
-			delete [] aPosValues;
+	int values[6];
+	for (size_t i = 0; i < 6; ++i) {
+		values[i] = fz::to_integral(tokens[i], std::numeric_limits<int>::min());
+		if (values[i] == std::numeric_limits<int>::min()) {
 			return false;
 		}
 	}
 
-	m_pTopSplitter->SetSashPosition(aPosValues[0]);
+	m_pTopSplitter->SetSashPosition(values[0]);
 
-	m_pBottomSplitter->SetSashPosition(aPosValues[1]);
+	m_pBottomSplitter->SetSashPosition(values[1]);
 
 	CContextControl::_context_controls* controls = m_pContextControl ? m_pContextControl->GetCurrentControls() : 0;
 	if (!controls) {
-		delete [] aPosValues;
 		return false;
 	}
 
-	double pos = (double)aPosValues[2] / 1000000000;
-	if (pos >= 0 && pos <= 1)
+	double pos = static_cast<double>(values[2]) / 1000000000;
+	if (pos >= 0 && pos <= 1) {
 		controls->pViewSplitter->SetRelativeSashPosition(pos);
+	}
 
-	controls->pLocalSplitter->SetSashPosition(aPosValues[3]);
-	controls->pRemoteSplitter->SetSashPosition(aPosValues[4]);
+	controls->pLocalSplitter->SetSashPosition(values[3]);
+	controls->pRemoteSplitter->SetSashPosition(values[4]);
 
-	pos = (double)aPosValues[5] / 1000000000;
+	pos = static_cast<double>(values[5]) / 1000000000;
 	if (pos >= 0 && pos <= 1) {
 		m_pQueueLogSplitter->SetRelativeSashPosition(pos);
 	}
-	delete [] aPosValues;
 
 	return true;
 }
@@ -2314,7 +2315,7 @@ void CMainFrame::OnActivate(wxActivateEvent& event)
 		return;
 	}
 
-#ifdef __WXMAC__
+#if defined(__WXMAC__) && !wxCHECK_VERSION(3, 1, 0)
 	// wxMac looses focus information if the window becomes inactive.
 	// Restore focus to the previously focused child, otherwise focus ends up
 	// in the quickconnect bar.
@@ -2353,7 +2354,7 @@ void CMainFrame::OnToolbarComparison(wxCommandEvent&)
 			return;
 		}
 
-		if (controls->pLocalListSearchPanel->IsShown() || controls->pRemoteListSearchPanel->IsShown()) {
+		if ((controls->pLocalListSearchPanel && controls->pLocalListSearchPanel->IsShown()) || (controls->pRemoteListSearchPanel && controls->pRemoteListSearchPanel->IsShown())) {
 			CConditionalDialog dlg(this, CConditionalDialog::quick_search, CConditionalDialog::yesno);
 			dlg.SetTitle(_("Directory comparison"));
 			dlg.AddText(_("To compare directories quick search must be closed."));
@@ -2362,11 +2363,11 @@ void CMainFrame::OnToolbarComparison(wxCommandEvent&)
 				return;
 			}
 
-			if (controls->pLocalListSearchPanel->IsShown()) {
+			if (controls->pLocalListSearchPanel && controls->pLocalListSearchPanel->IsShown()) {
 				controls->pLocalListSearchPanel->Close();
 			}
 
-			if (controls->pRemoteListSearchPanel->IsShown()) {
+			if (controls->pRemoteListSearchPanel && controls->pRemoteListSearchPanel->IsShown()) {
 				controls->pRemoteListSearchPanel->Close();
 			}
 		}
@@ -2531,30 +2532,28 @@ void CMainFrame::ProcessCommandLine()
 	if (!param.empty()) {
 		std::wstring error;
 
-		ServerWithCredentials server;
+		Site site;
 
 		wxString logontype = pCommandLine->GetOption(CCommandLine::logontype);
 		if (logontype == _T("ask")) {
-			server.SetLogonType(LogonType::ask);
+			site.SetLogonType(LogonType::ask);
 		}
 		else if (logontype == _T("interactive")) {
-			server.SetLogonType(LogonType::interactive);
+			site.SetLogonType(LogonType::interactive);
 		}
 
 		CServerPath path;
-		if (!server.ParseUrl(param, 0, std::wstring(), std::wstring(), error, path)) {
+		if (!site.ParseUrl(param, 0, std::wstring(), std::wstring(), error, path)) {
 			wxString str = _("Parameter not a valid URL");
 			str += _T("\n") + error;
 			wxMessageBoxEx(error, _("Syntax error in command line"));
 		}
 
-		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) && server.credentials.logonType_ == LogonType::normal) {
-			server.SetLogonType(LogonType::ask);
-			CLoginManager::Get().RememberPassword(server);
+		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) && site.credentials.logonType_ == LogonType::normal) {
+			site.SetLogonType(LogonType::ask);
+			CLoginManager::Get().RememberPassword(site);
 		}
 
-		Site site;
-		site.server_ = server;
 		Bookmark bm;
 		bm.m_remoteDir = path;
 		ConnectToSite(site, bm);
