@@ -12,6 +12,7 @@
 
 #include <wx/animate.h>
 #include <wx/hyperlink.h>
+#include <wx/statline.h>
 
 BEGIN_EVENT_TABLE(CUpdateDialog, wxDialogEx)
 EVT_BUTTON(XRCID("ID_INSTALL"), CUpdateDialog::OnInstall)
@@ -60,10 +61,36 @@ int CUpdateDialog::ShowModal()
 		return wxID_CANCEL;
 	}
 
-	if (!Load(parent_, _T("ID_UPDATE_DIALOG"))) {
+	if (!Create(parent_, wxID_ANY, _("Check for Updates"))) {
 		return wxID_CANCEL;
 	}
 
+
+	auto const& lay = layout();
+	auto outer = new wxBoxSizer(wxVERTICAL);
+	SetSizer(outer);
+
+	auto main = lay.createFlex(1);
+	outer->Add(main, 1, wxALL|wxGROW, lay.border);
+
+	main->AddGrowableCol(0);
+	main->AddGrowableRow(0);
+
+	main->Add(new wxPanel(this, XRCID("ID_CONTENT")), 1, wxGROW);
+	main->Add(new wxStaticLine(this), lay.grow);
+
+	auto buttons = new wxBoxSizer(wxHORIZONTAL);
+	main->Add(buttons, 0, wxALIGN_RIGHT);
+
+	bool debug = false;
+	wxString v;
+	if (wxGetEnv(_T("FZDEBUG"), &v) && v == _T("1")) {
+		debug = true;
+	}
+	buttons->Add(new wxButton(this, XRCID("ID_DEBUGLOG"), L"show log"))->Show(debug);
+	buttons->Add(new wxButton(this, wxID_CANCEL, _("&Close")));
+
+	InitXrc(L"update.xrc");
 	LoadPanel(_T("ID_CHECKING_PANEL"));
 	LoadPanel(_T("ID_FAILURE_PANEL"));
 	LoadPanel(_T("ID_NEWVERSION_PANEL"));
@@ -73,12 +100,18 @@ int CUpdateDialog::ShowModal()
 	}
 
 	wxAnimation a = CThemeProvider::Get()->CreateAnimation(_T("ART_THROBBER"), wxSize(16,16));
-	XRCCTRL(*this, "ID_WAIT_CHECK", wxAnimationCtrl)->SetMinSize(a.GetSize());
-	XRCCTRL(*this, "ID_WAIT_CHECK", wxAnimationCtrl)->SetAnimation(a);
-	XRCCTRL(*this, "ID_WAIT_CHECK", wxAnimationCtrl)->Play();
-	XRCCTRL(*this, "ID_WAIT_DOWNLOAD", wxAnimationCtrl)->SetMinSize(a.GetSize());
-	XRCCTRL(*this, "ID_WAIT_DOWNLOAD", wxAnimationCtrl)->SetAnimation(a);
-	XRCCTRL(*this, "ID_WAIT_DOWNLOAD", wxAnimationCtrl)->Play();
+	auto ctrl = XRCCTRL(*this, "ID_WAIT_CHECK", wxAnimationCtrl);
+	if (ctrl) {
+		ctrl->SetMinSize(a.GetSize());
+		ctrl->SetAnimation(a);
+		ctrl->Play();
+	}
+	ctrl = XRCCTRL(*this, "ID_WAIT_DOWNLOAD", wxAnimationCtrl);
+	if (ctrl) {
+		ctrl->SetMinSize(a.GetSize());
+		ctrl->SetAnimation(a);
+		ctrl->Play();
+	}
 
 	InitFooter();
 
@@ -126,7 +159,7 @@ void CUpdateDialog::InitFooter()
 
 			wxXmlResource res(wxXRC_NO_RELOADING);
 			InitHandlers(res);
-			if (res.Load(_T("blob:") + resources)) {
+			if (res.Load(_T("blob64:") + resources)) {
 				auto sizer = xrc_call(*this, "ID_NEWVERSION_PANEL", &wxPanel::GetSizer);
 				if (sizer) {
 					wxPanel* p{};
@@ -152,6 +185,10 @@ void CUpdateDialog::InitFooter()
 void CUpdateDialog::Wrap()
 {
 	wxPanel* parentPanel = XRCCTRL(*this, "ID_CONTENT", wxPanel);
+	if (!parentPanel) {
+		return;
+	}
+
 	wxSize canvas;
 	canvas.x = GetSize().x - parentPanel->GetSize().x;
 	canvas.y = GetSize().y - parentPanel->GetSize().y;
@@ -228,7 +265,7 @@ void CUpdateDialog::UpdaterStateChanged( UpdaterState s, build const& v )
 		panels_[pagenames::checking]->Show();
 	}
 	else if( s == UpdaterState::newversion || s == UpdaterState::newversion_ready || s == UpdaterState::newversion_downloading ) {
-		XRCCTRL(*this, "ID_VERSION", wxStaticText)->SetLabel(v.version_);
+		xrc_call(*this, "ID_VERSION", &wxStaticText::SetLabel, v.version_);
 
 		wxString news = updater_.GetChangelog();
 
@@ -239,17 +276,16 @@ void CUpdateDialog::UpdaterStateChanged( UpdaterState s, build const& v )
 
 		XRCCTRL(*this, "ID_NEWS_LABEL", wxStaticText)->Show(!news.empty());
 		XRCCTRL(*this, "ID_NEWS", wxTextCtrl)->Show(!news.empty());
-		if( news != XRCCTRL(*this, "ID_NEWS", wxTextCtrl)->GetValue() ) {
+		if (news != XRCCTRL(*this, "ID_NEWS", wxTextCtrl)->GetValue()) {
 			XRCCTRL(*this, "ID_NEWS", wxTextCtrl)->ChangeValue(news);
 		}
 		bool downloading = s == UpdaterState::newversion_downloading;
 		XRCCTRL(*this, "ID_DOWNLOAD_LABEL", wxStaticText)->Show(downloading);
 		XRCCTRL(*this, "ID_WAIT_DOWNLOAD", wxAnimationCtrl)->Show(downloading);
 		XRCCTRL(*this, "ID_DOWNLOAD_PROGRESS", wxStaticText)->Show(downloading);
-		if( downloading ) {
+		if (downloading) {
 			timer_.Start(500);
-			wxTimerEvent ev;
-			OnTimer(ev);
+			UpdateProgress();
 		}
 
 		bool ready = s == UpdaterState::newversion_ready;
@@ -301,6 +337,11 @@ void CUpdateDialog::OnInstall(wxCommandEvent&)
 }
 
 void CUpdateDialog::OnTimer(wxTimerEvent&)
+{
+	UpdateProgress();
+}
+
+void CUpdateDialog::UpdateProgress()
 {
 	int64_t size = updater_.AvailableBuild().size_;
 	int64_t downloaded = updater_.BytesDownloaded();
