@@ -15,6 +15,8 @@
 #include <libfilezilla/file.hpp>
 #include <libfilezilla/hash.hpp>
 #include <libfilezilla/local_filesys.hpp>
+#include <libfilezilla/signature.hpp>
+#include <libfilezilla/translate.hpp>
 
 #include <string>
 
@@ -301,7 +303,7 @@ int CUpdater::Request(fz::uri const& uri)
 	pending_commands_.emplace_back(new CDisconnectCommand);
 
 	CServer server(fz::equal_insensitive_ascii(uri.scheme_, std::string("http")) ? HTTP : HTTPS, DEFAULT, fz::to_wstring_from_utf8(uri.host_), uri.port_);
-	pending_commands_.emplace_back(new CConnectCommand(server, Credentials()));
+	pending_commands_.emplace_back(new CConnectCommand(server, ServerHandle(), Credentials()));
 	pending_commands_.emplace_back(new CHttpRequestCommand(uri));
 
 	return ContinueDownload();
@@ -328,14 +330,14 @@ int CUpdater::ContinueDownload()
 
 bool CUpdater::CreateConnectCommand(std::wstring const& url)
 {
-	ServerWithCredentials s;
+	Site s;
 	CServerPath path;
 	std::wstring error;
 	if (!s.ParseUrl(url, 0, std::wstring(), std::wstring(), error, path) || (s.server.GetProtocol() != HTTP && s.server.GetProtocol() != HTTPS)) {
 		return false;
 	}
 
-	pending_commands_.emplace_back(new CConnectCommand(s.server, s.credentials));
+	pending_commands_.emplace_back(new CConnectCommand(s.server, s.Handle(), s.credentials));
 	return true;
 }
 
@@ -344,7 +346,7 @@ bool CUpdater::CreateTransferCommand(std::wstring const& url, std::wstring const
 	CFileTransferCommand::t_transferSettings transferSettings;
 	transferSettings.fsync = true;
 
-	ServerWithCredentials s;
+	Site s;
 	CServerPath path;
 	std::wstring error;
 	if (!s.ParseUrl(url, 0, std::wstring(), std::wstring(), error, path) || (s.server.GetProtocol() != HTTP && s.server.GetProtocol() != HTTPS)) {
@@ -398,19 +400,19 @@ void CUpdater::ProcessNotification(std::unique_ptr<CNotification> && notificatio
 			else if (pData->GetRequestID() == reqId_certificate) {
 				auto & certNotification = static_cast<CCertificateNotification &>(*pData.get());
 				if (m_use_internal_rootcert) {
-					auto certs = certNotification.GetCertificates();
+					auto certs = certNotification.info_.get_certificates();
 					if (certs.size() > 1) {
 						auto const& ca = certs.back();
-						std::vector<uint8_t> ca_data = ca.GetRawData();
+						std::vector<uint8_t> ca_data = ca.get_raw_data();
 
 						std::string updater_root = fz::base64_decode(s_update_cert);
 						if (ca_data.size() == updater_root.size() && !memcmp(&ca_data[0], updater_root.c_str(), ca_data.size())) {
-							certNotification.m_trusted = true;
+							certNotification.trusted_ = true;
 						}
 					}
 				}
 				else {
-					certNotification.m_trusted = true;
+					certNotification.trusted_ = true;
 				}
 			}
 			engine_->SetAsyncRequestReply(std::move(pData));
@@ -726,11 +728,34 @@ void CUpdater::ParseData()
 			auto const size = fz::to_integral<uint64_t>(sizestr);
 			if (!size) {
 				if (COptions::Get()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL) == 4) {
-					log_ += fz::sprintf(L"Could not parse size: %s", sizestr) + L"\n";
+					log_ += fz::sprintf(L"Could not parse size: %s\n", sizestr);
 				}
 				continue;
 			}
 
+			bool valid_signature{};
+			for (size_t i = 6; i < tokens.size(); ++i) {
+				auto const& token = tokens[6];
+				if (token.substr(0, 4) == "sig:") {
+					auto const& sig = token.substr(4);
+					auto raw_sig = fz::base64_decode(fz::to_utf8(sig));
+					auto raw_hash = fz::hex_decode(hash);
+
+					// Append the version to the file hash to protect against replays
+					raw_hash.push_back(0);
+					raw_hash.insert(raw_hash.cend(), versionOrDate.cbegin(), versionOrDate.cend());
+
+					if (!raw_sig.empty() || !raw_hash.empty()) {
+						auto const pub = fz::public_verification_key::from_base64("xrjuitldZT7pvIhK9q1GVNfptrepB/ctt5aK1QO5RaI");
+						valid_signature = fz::verify(std::string_view(reinterpret_cast<char const*>(raw_hash.data()), raw_hash.size()), raw_sig, pub);
+					}
+				}
+			}
+			if (!valid_signature) {
+				log_ += fz::sprintf(L"Ignoring line with inalid or missing signature for hash %s\n", hash);
+				continue;
+			}
+
 			b.url_ = url;
 			b.size_ = size;
 			b.hash_ = fz::str_tolower_ascii(hash);
